==> eav_field-2.0.0.tar.gz: eav_field/src/Plugin/Field/FieldType/EavItemList.php <==
<?php

namespace Drupal\eav_field\Plugin\Field\FieldType;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Field\EntityReferenceFieldItemList;
use Drupal\Core\Form\FormStateInterface;
use Drupal\eav_field\Entity\EavValue;

class EavItemList extends EntityReferenceFieldItemList {

  /**
   * {@inheritdoc}
   */
  public function defaultValuesForm(array &$form, FormStateInterface $form_state) {}

  /**
   * {@inheritdoc}
   */
  public function postSave($update) {
    // Delete orhpaned eav_value entities
    if ($update) {
      $entity = $this->getEntity();
      $original_entity = $entity->original; /** @var EntityInterface $original_entity */
      $new_eav_value_ids = $this->getValueEntitiesIds();
      $eav_field_name = $this->getFieldDefinition()->getName();
      $original_eav_field_items = $original_entity->get($eav_field_name); /** @var EavItemList $original_eav_field_items */

      foreach ($original_eav_field_items->referencedEntities() as $original_eav_value_entity) {
        if (!in_array($original_eav_value_entity->id(), $new_eav_value_ids)) {
          $original_eav_value_entity->delete();
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function delete() {
    /** @var EavValue $eav_value_entity */
    foreach ($this->referencedEntities() as $eav_value_entity) {
      $eav_value_entity->delete();
    }

    parent::delete();
  }

  /**
   * Return field item by attribute id or machine name.
   *
   * @param mixed $attribute Attribute id or machine name
   *
   * @return EavItem
   */
  public function getItemByAttribute($attribute) {
    $attribute_id = is_numeric($attribute) ? $attribute : NULL;
    $attribute_name = !is_numeric($attribute) ? $attribute : NULL;

    /** @var EavItem $eav_item */
    foreach ($this as $eav_item) {
      $eav_value = $eav_item->getValueEntity();

      // By id
      if ($attribute_id) {
        if ($attribute_id == $eav_value->getAttributeId()) {
          return $eav_item;
        }
      }
      // By machine name
      else {
        $attribute = $eav_value->getAttributeEntity();
        if ($attribute && $attribute_name == $attribute->getMachineName()) {
          return $eav_item;
        }
      }
    }
  }

  /**
   * Return eav_value entities ids.
   */
  public function referencedEntitiesIds() {
    return array_filter(array_map(function (EntityInterface $entity) {
      return $entity->id();
    }, $this->referencedEntities()));
  }

  /**
   * Return eav_value entities ids.
   */
  public function getValueEntitiesIds() {
    return $this->referencedEntitiesIds();
  }

}

==> eav_field-2.0.1.tar.gz: eav_field/src/Plugin/Field/FieldType/EavItemList.php <==
<?php

namespace Drupal\eav_field\Plugin\Field\FieldType;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Field\EntityReferenceFieldItemList;
use Drupal\Core\Form\FormStateInterface;
use Drupal\eav_field\Entity\EavValue;

class EavItemList extends EntityReferenceFieldItemList {

  /**
   * {@inheritdoc}
   */
  public function defaultValuesForm(array &$form, FormStateInterface $form_state) {}

  /**
   * {@inheritdoc}
   */
  public function postSave($update) {
    // Delete orhpaned eav_value entities
    if ($update) {
      $entity = $this->getEntity();
      $original_entity = $entity->original; /** @var EntityInterface $original_entity */
      $new_eav_value_ids = $this->getValueEntitiesIds();
      $eav_field_name = $this->getFieldDefinition()->getName();
      $original_eav_field_items = $original_entity->get($eav_field_name); /** @var EavItemList $original_eav_field_items */

      foreach ($original_eav_field_items->referencedEntities() as $original_eav_value_entity) {
        if (!in_array($original_eav_value_entity->id(), $new_eav_value_ids)) {
          $original_eav_value_entity->delete();
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function delete() {
    /** @var EavValue $eav_value_entity */
    foreach ($this->referencedEntities() as $eav_value_entity) {
      $eav_value_entity->delete();
    }

    parent::delete();
  }

  /**
   * Return field item by attribute id or machine name.
   *
   * @param mixed $attribute Attribute id or machine name
   */
  public function getItemByAttribute($attribute): ?EavItem {
    $attribute_id = is_numeric($attribute) ? $attribute : NULL;
    $attribute_name = !is_numeric($attribute) ? $attribute : NULL;

    /** @var EavItem $eav_item */
    foreach ($this as $eav_item) {
      $eav_value = $eav_item->getValueEntity();

      // By id
      if ($attribute_id) {
        if ($attribute_id == $eav_value->getAttributeId()) {
          return $eav_item;
        }
      }
      // By machine name
      else {
        $attribute = $eav_value->getAttributeEntity();
        if ($attribute && $attribute_name == $attribute->getMachineName()) {
          return $eav_item;
        }
      }
    }
  }

  /**
   * Return eav_value entities ids.
   */
  public function referencedEntitiesIds(): array {
    return array_filter(array_map(function (EntityInterface $entity) {
      return $entity->id();
    }, $this->referencedEntities()));
  }

  /**
   * Return eav_value entities ids.
   */
  public function getValueEntitiesIds(): array {
    return $this->referencedEntitiesIds();
  }

}
